import os

import typer
import yaml


def convert_to_env_vars(d, prefix=""):
    env_vars = {}

    for key, value in d.items():
        if isinstance(value, dict):
            env_vars.update(convert_to_env_vars(value, prefix + key.upper() + "_"))
        else:
            env_vars[prefix + key.upper()] = value

    return env_vars


def load_yaml_env_vars():
    home_dir = os.path.expanduser("~")
    yml_file_path = os.path.join(home_dir, ".config", "hacli", "setting.yaml")

    if os.path.exists(yml_file_path):
        with open(yml_file_path, 'r') as file:
            config = yaml.safe_load(file)
            env_vars = convert_to_env_vars(config)
            for key, value in env_vars.items():
                os.environ[key] = value
                print(key, value)
    else:
        print(f"File {yml_file_path} does not exist.")
        raise typer.Exit(1)



from typing import Optional, List

import typer
from typing_extensions import Annotated

from ..tag.get_global_rollout_status import global_rollout_status_internally
from ..utils.ado import execute_pipeline

app = typer.Typer()

product_tag_mappings = {
    "pm": "pm",
    "sm": "sm",
    "um": "um",
    "vm": "vm",
    "sys": "st"
}


@app.command(name="deploy_inactive_services")
def deploy_inactive_services(release: str,
                             product_names: Annotated[Optional[List[str]], typer.Option("--product")] = None,
                             deploy_all_product: Annotated[bool, typer.Option("--all-product")] = False):
    tags_to_deploy: dict[str, str] | None = global_rollout_status_internally()
    if not tags_to_deploy:
        typer.echo("No tags found from global services deployment")
        return

    if not deploy_all_product and not product_names:
        typer.secho(f"Either specify product name or specify deploy all products.", fg=typer.colors.RED)
        return

    deploy_product = product_names if not deploy_all_product else list(product_tag_mappings.keys())
    valid_deploy_product = {product for product in deploy_product if product in product_tag_mappings}

    for product in valid_deploy_product:
        product_name = product.upper()
        tag = tags_to_deploy[product_tag_mappings[product.lower()].upper()]
        execute_pipeline(release=release,
                         pipeline_name_env_key="DEPLOY_PIPELINES_DEPLOY_INACTIVE_SERVICES_NAME",
                         pipeline_parameter_env_key="DEPLOY_PIPELINES_DEPLOY_INACTIVE_SERVICES_PARAMETERS",
                         product=product_name, tag=tag)



import typer

from ..utils.ado import execute_pipeline

app = typer.Typer()

service_name_abbr_list = [
    "pm",
    "sm",
    "um",
    "vm",
    "shared",  # TODO..
]


@app.command(name="update_active_service_config")
def update_active_service_config(release: str, product: str, service: str):
    execute_pipeline(release=release,
                     pipeline_name_env_key="DEPLOY_PIPELINES_UPDATE_ACTIVE_SERVICE_CONFIG_NAME",
                     pipeline_parameter_env_key="DEPLOY_PIPELINES_UPDATE_ACTIVE_SERVICE_CONFIG_PARAMETERS",
                     product=product.upper(),
                     service=service)



import typer

from ..utils.ado import execute_pipeline

app = typer.Typer()


@app.command(name="build_up_inactive_cluster")
def build_up_inactive_cluster(release: str):
    execute_pipeline(release=release,
                     pipeline_name_env_key="DEPLOY_PIPELINES_BUILD_UP_INACTIVE_CLUSTER_NAME",
                     pipeline_parameter_env_key="DEPLOY_PIPELINES_BUILD_UP_INACTIVE_CLUSTER_PARAMETERS")



import typer

from ..utils.ado import execute_pipeline

app = typer.Typer()


@app.command(name="deploy_inactive_shared_services")
def deploy_inactive_shared_services(release: str, tag: str):
    execute_pipeline(release=release,
                     pipeline_name_env_key="DEPLOY_PIPELINES_DEPLOY_INACTIVE_SHARED_SERVICES_NAME",
                     pipeline_parameter_env_key="DEPLOY_PIPELINES_DEPLOY_INACTIVE_SHARED_SERVICES_PARAMETERS",
                     tag=tag)



import json
import os
import re

import typer
from azure.devops.v7_0.dashboard import Widget
from bs4 import BeautifulSoup
from markdown_it import MarkdownIt
from rich import print

from ..utils.ado import AzureDevOpsClient

app = typer.Typer()


def global_rollout_status_internally() -> dict[str, str] | None:
    client = AzureDevOpsClient(os.environ["PROJECT_GLOBAL_NAME"])

    tag_team = os.environ["TAG_TEAM"]
    tag_dashboard_id = os.environ["TAG_DASHBOARD_ID"]
    tag_active_cluster_widget_id = os.environ["TAG_ACTIVE_CLUSTER_WIDGET_ID"]
    tag_services_tag_widget_id = os.environ["TAG_SERVICES_TAG_WIDGET_ID"]

    widget: Widget = client.get_widget_from_dashboard(team=tag_team,
                                                      dashboard_id=tag_dashboard_id,
                                                      widget_id=tag_active_cluster_widget_id)
    green_clusters = [
        re.sub(r"[`]", "", match[0].strip())
        for match in re.findall(r"\| (.*?) \| (.*?) \| (.*?) \|", widget.settings)
        if "green" in match[1].lower() and "green" in match[2].lower()
    ]

    if green_clusters:
        active_cluster = green_clusters[0]
        tag_widget: Widget = client.get_widget_from_dashboard(team=tag_team,
                                                              dashboard_id=tag_dashboard_id,
                                                              widget_id=tag_services_tag_widget_id)
        md = MarkdownIt('js-default')
        html_content = md.render(tag_widget.settings)
        soup = BeautifulSoup(html_content, 'html.parser')

        result_map = {}
        for table in soup.find_all('table'):
            cluster_name = table.find('code').text.strip()
            cluster_data = {row.find_all('td')[0].text.strip(): row.find_all('td')[1].text.strip()
                            for row in table.find_all('tr')[1:]}

            result_map[cluster_name] = cluster_data
        return result_map.get(active_cluster)


@app.command(name="get_global_rollout_status.py")
def get_global_rollout_status():
    internally = global_rollout_status_internally()
    if internally:
        print("Got tags for services\n")
        print(json.dumps(internally, indent=2))
    else:
        msg = typer.style("No Active cluster Found", fg=typer.colors.RED, bold=True)
        typer.echo(msg)



import typer

from ..utils.ado import execute_pipeline

app = typer.Typer()


@app.command(name="purge_local_images")
def purge_local_images():
    execute_pipeline(release="master",
                     pipeline_name_env_key="DEPLOY_PIPELINES_PURGE_LOCAL_IMAGES_NAME",
                     pipeline_parameter_env_key="DEPLOY_PIPELINES_PURGE_LOCAL_IMAGES_PARAMETERS")



import typer

from ..utils.ado import execute_pipeline

app = typer.Typer()


@app.command(name="import_global_external_images")
def import_global_external_images():
    execute_pipeline(release="master",
                     pipeline_name_env_key="DEPLOY_PIPELINES_IMPORT_GLOBAL_EXTERNAL_IMAGES_NAME",
                     pipeline_parameter_env_key="DEPLOY_PIPELINES_IMPORT_GLOBAL_EXTERNAL_IMAGES_PARAMETERS",
                     project_name_env_key="PROJECT_GLOBAL_NAME",
                     project_repo_env_key="PROJECT_GLOBAL_REPO")



import json
from string import Template

import typer
from azure.containerregistry import ContainerRegistryClient
from azure.core.exceptions import ResourceNotFoundError
from azure.identity import DefaultAzureCredential

from ..tag.get_global_rollout_status import global_rollout_status_internally

app = typer.Typer()

default_services = {
    "pm": "product-management",
    "sm": "service-management",
    "um": "user-management",
    "vm": "vehicle-management",
    "st": "system"
}

import os


def get_service_image_names(parent_dir):
    directories_with_cn = []
    for subdir, dirs, files in os.walk(parent_dir):
        if 'cn' in dirs:
            directories_with_cn.append(os.path.basename(subdir))
    return directories_with_cn


@app.command(name="check_service_image_imported")
def check_service_image_imported():
    rollout_tags: dict[str, str] | None = global_rollout_status_internally()
    if rollout_tags is None:
        typer.secho("Global rollout status is None", fg=typer.colors.RED)
        return

    project_dir = os.environ["PROJECT_LOCAL_GIT_LOCAL_WORKING_DIR"]
    service_configuration_parent_dir = os.environ["SERVICES_CONFIGURATION_DIR"]
    service_configuration_setting_template_dir = os.environ["SERVICES_DEPLOYMENT_SETTING_DIR_TEMPLATE"]
    deployment_setting_service_key_template = os.environ["SERVICES_DEPLOYMENT_SETTING_SERVICE_KEY_TEMPLATE"]
    ENDPOINT = os.environ["PROJECT_LOCAL_ACR_URL"]

    credential = DefaultAzureCredential()
    with ContainerRegistryClient(ENDPOINT, credential) as client:
        for key, value in rollout_tags.items():
            service = default_services.get(key.lower())
            service_folder = os.path.join(project_dir, service_configuration_parent_dir, service)
            services_from_configuration = get_service_image_names(service_folder)
            service_configuration_setting_file = os.path.join(project_dir, Template(
                service_configuration_setting_template_dir).safe_substitute(service=service))

            with open(service_configuration_setting_file, "r") as f:
                loads: dict = json.loads(f.read())
                substitute = Template(deployment_setting_service_key_template).safe_substitute(service=service)
                get = loads.get(substitute)
                if get.get("services"):
                    services_from_configuration = [item for item in services_from_configuration if
                                                   item in get.get("services")]
            check_pass = True
            for service_module in services_from_configuration:
                try:
                    client.get_tag_properties(repository=f"service/{service_module}", tag=value)
                except ResourceNotFoundError:
                    check_pass = False
                    typer.secho(f"Service: {service_module}, Tag: '{value}' is not existed!", fg=typer.colors.RED)
                except Exception as e:
                    check_pass = False
                    typer.secho(f"Service: {service_module}, Tag: '{value}' checks with exception", fg=typer.colors.RED)

            if check_pass:
                typer.secho(f"Service: {service}, Tag: '{value}' , all images imported!", fg=typer.colors.GREEN)



import typer
from rich import print

from ..tag.get_global_rollout_status import global_rollout_status_internally
from ..utils.ado import execute_pipeline

app = typer.Typer()

tag_product_services = {
    "pm": "product-management",
    "sm": "service-management",
    "um": "user-management",
    "vm": "vehicle-management",
    "st": "system",
    "shared": "shared"
}


@app.command(name="import_global_service_images")
def import_global_service_images():
    confirm = typer.confirm("""Are you sure to trigger pipeline to import service images, 
     have you checked service images imported?""")
    if not confirm:
        return

    rollout_tags: dict[str, str] | None = global_rollout_status_internally()
    if rollout_tags is None:
        typer.secho("Global rollout status is None", fg=typer.colors.RED)
        return

    res = {}
    for key, value in tag_product_services.items():
        tag = rollout_tags.get(key.upper(), None)
        if tag:
            execute_pipeline(release="master",
                             pipeline_name_env_key="DEPLOY_PIPELINES_IMPORT_GLOBAL_SERVICE_IMAGES_NAME",
                             pipeline_parameter_env_key="DEPLOY_PIPELINES_IMPORT_GLOBAL_SERVICE_IMAGES_PARAMETERS",
                             project_name_env_key="PROJECT_GLOBAL_NAME",
                             project_repo_env_key="PROJECT_GLOBAL_REPO", product=value, tag=tag)
            res[value] = f"Trigger pipeline to import global service images for {tag}"
        else:
            res[value] = f"Not tag found for product {value}"

    print(res)



import os
from string import Template

import typer
from git import GitCommandError
from rich import print

from ..utils.git_commands import LocalGitCommand

app = typer.Typer()

default_services = [
    "product-management",
    "service-management",
    "vehicle-management",
    "user-management",
    "system"
]

from rich import print


@app.command(name="create_release_note_md")
def create_release_note_md(pre_release: str, cur_release: str):
    command = LocalGitCommand(True).repo

    #  TODO services change requests needed...
    str_list = []
    for service in default_services:
        commands = (
            Template(os.environ["GIT_RELEASE_NOTE_COMMAND_CONFIGURATION"])
            .safe_substitute(service=service,
                             pre_release=pre_release,
                             cur_release=cur_release)
        )
        try:
            str_list.append(f"\n### {service}\n")
            log_output = command.git.execute(command=commands, shell=True)
            str_list.append(log_output)
        except GitCommandError as e:
            str_list.append(e.stderr)

    final_str = '\n'.join(str_list)
    print(final_str)



import typer

from ..utils.ado import execute_pipeline

app = typer.Typer()


@app.command(name="sync_global_repo")
def sync_global_repo():
    execute_pipeline(release="master",
                     pipeline_name_env_key="DEPLOY_PIPELINES_SYNC_GLOBAL_REPO_NAME",
                     pipeline_parameter_env_key="DEPLOY_PIPELINES_SYNC_GLOBAL_REPO_PARAMETERS",
                     project_name_env_key="PROJECT_GLOBAL_NAME",
                     project_repo_env_key="PROJECT_GLOBAL_REPO")



import os

import git


class LocalGitCommand:
    def __init__(self, is_global: bool):
        self.is_local = is_global
        self.working_dir = os.environ["PROJECT_GLOBAL_GIT_LOCAL_WORKING_DIR"] if is_global else os.environ[
            "PROJECT_LOCAL_GIT_LOCAL_WORKING_DIR"]
        self.repo = git.Repo(self.working_dir)

    def create_branch(self, branch: str, base_branch):
        if self.repo.active_branch.name != base_branch:
            self.repo.git.checkout(base_branch)
            print(f"Switched to base branch '{base_branch}'!")
        self.repo.git.pull('origin', base_branch)
        self.repo.git.checkout("-b", branch)



import json
from os import environ
from string import Template

import typer
from azure.devops.connection import Connection
from azure.devops.exceptions import AzureDevOpsServiceError
from azure.devops.v7_0.dashboard import TeamContext, Dashboard, DashboardClient, Widget
from azure.devops.v7_0.git import GitClient
from azure.devops.v7_0.pipelines import PipelinesClient, Pipeline, Run
from msrest.authentication import BasicAuthentication


class AzureDevOpsClient:
    def __init__(self, project):
        self.org_url = environ["ORG_URL"]
        self.personal_access_token = environ["PERSONAL_ACCESS_TOKEN"]
        self.project = project
        self.credentials = BasicAuthentication('', self.personal_access_token)
        self.connection = Connection(base_url=self.org_url, creds=self.credentials)

        self.pipeline_client: PipelinesClient = self.connection.clients.get_pipelines_client()
        self.dashboard_client: DashboardClient = self.connection.clients.get_dashboard_client()
        self.git_client: GitClient = self.connection.clients.get_git_client()

    def trigger_pipeline(self, pipeline_id: int, run_parameters: dict) -> Run:
        return self.pipeline_client.run_pipeline(pipeline_id=pipeline_id,
                                                 project=self.project,
                                                 run_parameters=run_parameters)

    def get_pipeline(self, pipeline_id: int) -> Pipeline:
        return self.pipeline_client.get_pipeline(self.project, pipeline_id=pipeline_id)

    def list_pipelines(self) -> list[Pipeline]:
        return self.pipeline_client.list_pipelines(self.project, top=9999)

    def get_pipeline_by_name(self, pipeline_name: str) -> Pipeline | None:
        pipelines: list[Pipeline] = self.pipeline_client.list_pipelines(self.project, top=9999)
        for pipeline in pipelines:
            if pipeline.name == pipeline_name:
                return pipeline

    def get_dashboard(self, team: str, dashboard_id: str) -> Dashboard:
        team_context = TeamContext(project=self.project, team=team)
        return self.dashboard_client.get_dashboard(team_context, dashboard_id=dashboard_id)

    def get_widget_from_dashboard(self, team: str, dashboard_id: str, widget_id: str) -> Widget:
        team_context = TeamContext(project=self.project, team=team)
        return self.dashboard_client.get_widget(team_context, dashboard_id=dashboard_id, widget_id=widget_id)

    def check_release_branch(self, repository_id: str, release_branch_name: str):
        try:
            self.git_client.get_branch(repository_id=repository_id, name=release_branch_name, project=self.project)
            return True
        except AzureDevOpsServiceError as ex:
            return False


def execute_pipeline(release: str,
                     pipeline_name_env_key: str,
                     pipeline_parameter_env_key: str,
                     project_name_env_key: str = "PROJECT_LOCAL_NAME",
                     project_repo_env_key: str = "PROJECT_LOCAL_REPO", **kwargs):
    kwargs["release"] = release

    client = AzureDevOpsClient(environ[project_name_env_key])

    release_branch_name = release if release == "master" else f"release/{release}-Application-Deployment"
    release_branch_exist: bool = client.check_release_branch(environ[project_repo_env_key], release_branch_name)
    if not release_branch_exist:
        typer.secho(f"Release branch {release_branch_name} does not exist", fg=typer.colors.RED)
        return

    pipeline_name = Template(environ[pipeline_name_env_key]).safe_substitute(**kwargs)
    pipeline = client.get_pipeline_by_name(pipeline_name)

    if not pipeline:
        typer.secho(f"Pipeline {pipeline_name} is not existed", fg=typer.colors.RED)
        return

    pipeline_parameter_template = environ[pipeline_parameter_env_key]
    run_parameters = json.loads(Template(pipeline_parameter_template).safe_substitute(**kwargs))
    confirm = typer.confirm(f"Are you sure to trigger pipeline {pipeline_name}?")
    if not confirm:
        return

    web_url: str = pipeline._links.additional_properties.get("web").get('href')
    typer.launch(web_url)
    client.trigger_pipeline(pipeline_id=pipeline.id, run_parameters=run_parameters)



import typer

from . import deploy
from . import git
from . import image
from . import tag
from .utils.envs import load_yaml_env_vars

app = typer.Typer(pretty_exceptions_show_locals=False)

app.add_typer(deploy.app, name="deploy")
app.add_typer(tag.app, name="tag")
app.add_typer(git.app, name="git")
app.add_typer(image.app, name="image")

load_yaml_env_vars()



import json
import os
from string import Template

import typer
from azure.identity import DefaultAzureCredential
from azure.keyvault.secrets import SecretClient
from rich import print

from ..tag.get_global_rollout_status import global_rollout_status_internally

app = typer.Typer()

tag_product_services = {
    "pm": "product-management",
    "sm": "service-management",
    "um": "user-management",
    "vm": "vehicle-management",
    "st": "system"
}


@app.command(name="check_services_deployment_status")
def check_services_deployment_status():
    rollout_tags: dict[str, str] | None = global_rollout_status_internally()
    if rollout_tags is None:
        typer.echo("Global rollout status is None")
        return

    key_vault_uri = os.environ["PROJECT_LOCAL_KEY_VAULT_URL"]

    credential = DefaultAzureCredential()
    client = SecretClient(vault_url=key_vault_uri, credential=credential)
    active_cluster_key = os.environ["PROJECT_LOCAL_ACTIVE_CLUSTER_KEY_VAULT_KEY"]
    active_cluster = client.get_secret(active_cluster_key).value

    res = {}
    for key, value in rollout_tags.items():
        product = tag_product_services.get(key.lower())
        product_tag_key = Template(os.environ["PROJECT_LOCAL_PRODUCT_DEPLOYED_KEY_VAULT_KEY"]).safe_substitute(
            active_cluster=active_cluster, product=product)
        tag = client.get_secret(product_tag_key).value
        res[product] = value == tag

    print(json.dumps(res, indent=2))



import typer
from typing_extensions import Annotated

from ..utils.git_commands import LocalGitCommand

app = typer.Typer()


#  TODO to complete it
@app.command(name="create_local_repo_branch")
def create_local_repo_branch(branch: str,
                             base_branch: str = "master",
                             is_global: Annotated[bool, typer.Option("--global")] = False,
                             push: Annotated[bool, typer.Option("--push")] = False):
    command = LocalGitCommand(is_global)
    command.create_branch(branch, base_branch)
